@@ -68,7 +68,7 @@ vips_sRGB2HSV_line(VipsColour *colour, VipsPel *out, VipsPel **in, int width)
 				c_max = p[0];
 				c_min = p[1];
 				secondary_diff = p[1] - p[2];
-				wrap_around_hue = 255.0;
+				wrap_around_hue = 255.0F;
 			}
 			else {
 				/* Center blue.
@@ -76,7 +76,7 @@ vips_sRGB2HSV_line(VipsColour *colour, VipsPel *out, VipsPel **in, int width)
 				c_max = p[2];
 				c_min = VIPS_MIN(p[1], p[0]);
 				secondary_diff = p[0] - p[1];
-				wrap_around_hue = 170.0;
+				wrap_around_hue = 170.0F;
 			}
 		}
 		else {
@@ -86,7 +86,7 @@ vips_sRGB2HSV_line(VipsColour *colour, VipsPel *out, VipsPel **in, int width)
 				c_max = p[0];
 				c_min = p[2];
 				secondary_diff = p[1] - p[2];
-				wrap_around_hue = 0.0;
+				wrap_around_hue = 0.0F;
 			}
 			else {
 				/* Center green
@@ -94,7 +94,7 @@ vips_sRGB2HSV_line(VipsColour *colour, VipsPel *out, VipsPel **in, int width)
 				c_max = p[1];
 				c_min = VIPS_MIN(p[2], p[0]);
 				secondary_diff = p[2] - p[0];
-				wrap_around_hue = 85.0;
+				wrap_around_hue = 85.0F;
 			}
 		}
 
@@ -155,15 +155,16 @@ vips_sRGB2HSV_init(VipssRGB2HSV *sRGB2HSV)
  * vips_sRGB2HSV: (method)
  * @in: input image
  * @out: (out): output image
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Convert to HSV.
  *
  * HSV is a crude polar coordinate system for RGB images. It is provided for
- * compatibility with other image processing systems. See vips_Lab2LCh() for a
+ * compatibility with other image processing systems. See [method@Image.Lab2LCh] for a
  * much better colour space.
  *
- * See also: vips_HSV2sRGB(), vips_Lab2LCh().
+ * ::: seealso
+ *     [method@Image.HSV2sRGB], [method@Image.Lab2LCh].
  *
  * Returns: 0 on success, -1 on error.
  */
