@@ -84,7 +84,7 @@ typedef struct _VipsSequential {
 
 	/* Lock access to y_pos with this.
 	 */
-	GMutex *lock;
+	GMutex lock;
 
 	/* The next read from our source will fetch this scanline, ie. it's 0
 	 * when we start.
@@ -102,13 +102,13 @@ typedef VipsConversionClass VipsSequentialClass;
 G_DEFINE_TYPE(VipsSequential, vips_sequential, VIPS_TYPE_CONVERSION);
 
 static void
-vips_sequential_dispose(GObject *gobject)
+vips_sequential_finalize(GObject *gobject)
 {
 	VipsSequential *sequential = (VipsSequential *) gobject;
 
-	VIPS_FREEF(vips_g_mutex_free, sequential->lock);
+	g_mutex_clear(&sequential->lock);
 
-	G_OBJECT_CLASS(vips_sequential_parent_class)->dispose(gobject);
+	G_OBJECT_CLASS(vips_sequential_parent_class)->finalize(gobject);
 }
 
 static int
@@ -126,14 +126,14 @@ vips_sequential_generate(VipsRegion *out_region,
 
 	VIPS_GATE_START("vips_sequential_generate: wait");
 
-	vips__worker_lock(sequential->lock);
+	vips__worker_lock(&sequential->lock);
 
 	VIPS_GATE_STOP("vips_sequential_generate: wait");
 
 	/* If we've seen an error, everything must stop.
 	 */
 	if (sequential->error) {
-		g_mutex_unlock(sequential->lock);
+		g_mutex_unlock(&sequential->lock);
 		return -1;
 	}
 
@@ -157,7 +157,7 @@ vips_sequential_generate(VipsRegion *out_region,
 			area.height = VIPS_MIN(sequential->tile_height, r->top - area.top);
 			if (vips_region_prepare(ir, &area)) {
 				sequential->error = -1;
-				g_mutex_unlock(sequential->lock);
+				g_mutex_unlock(&sequential->lock);
 				return -1;
 			}
 
@@ -172,13 +172,13 @@ vips_sequential_generate(VipsRegion *out_region,
 	if (vips_region_prepare(ir, r) ||
 		vips_region_region(out_region, ir, r, r->left, r->top)) {
 		sequential->error = -1;
-		g_mutex_unlock(sequential->lock);
+		g_mutex_unlock(&sequential->lock);
 		return -1;
 	}
 
 	sequential->y_pos = VIPS_MAX(sequential->y_pos, VIPS_RECT_BOTTOM(r));
 
-	g_mutex_unlock(sequential->lock);
+	g_mutex_unlock(&sequential->lock);
 
 	return 0;
 }
@@ -231,7 +231,7 @@ vips_sequential_class_init(VipsSequentialClass *class)
 
 	VIPS_DEBUG_MSG("vips_sequential_class_init\n");
 
-	gobject_class->dispose = vips_sequential_dispose;
+	gobject_class->finalize = vips_sequential_finalize;
 	gobject_class->set_property = vips_object_set_property;
 	gobject_class->get_property = vips_object_get_property;
 
@@ -270,7 +270,7 @@ vips_sequential_class_init(VipsSequentialClass *class)
 static void
 vips_sequential_init(VipsSequential *sequential)
 {
-	sequential->lock = vips_g_mutex_new();
+	g_mutex_init(&sequential->lock);
 	sequential->tile_height = 1;
 	sequential->error = 0;
 	sequential->trace = FALSE;
@@ -280,21 +280,21 @@ vips_sequential_init(VipsSequential *sequential)
  * vips_sequential: (method)
  * @in: input image
  * @out: (out): output image
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
- *
- * * @tile_height: height of cache strips
- *
- * This operation behaves rather like vips_copy() between images
+ * This operation behaves rather like [method@Image.copy] between images
  * @in and @out, except that it checks that pixels on @in are only requested
  * top-to-bottom. This operation is useful for loading file formats which are
  * strictly top-to-bottom, like PNG.
  *
  * @tile_height can be used to set the size of the tiles that
- * vips_sequential() uses. The default value is 1.
+ * [method@Image.sequential] uses. The default value is 1.
  *
- * See also: vips_cache(), vips_linecache(), vips_tilecache().
+ * ::: tip "Optional arguments"
+ *     * @tile_height: `gint`, height of cache strips
+ *
+ * ::: seealso
+ *     [method@Image.linecache], [method@Image.tilecache].
  *
  * Returns: 0 on success, -1 on error.
  */
