@@ -157,7 +157,10 @@ vips_foreign_save_jpeg_class_init(VipsForeignSaveJpegClass *class)
 
 	/* See also vips_foreign_save_tiff_build() when saving JPEG in TIFF.
 	 */
-	save_class->saveable = VIPS_SAVEABLE_RGB_CMYK;
+	save_class->saveable =
+		VIPS_FOREIGN_SAVEABLE_MONO |
+		VIPS_FOREIGN_SAVEABLE_RGB |
+		VIPS_FOREIGN_SAVEABLE_CMYK;
 	save_class->format_table = bandfmt_jpeg;
 
 	VIPS_ARG_INT(class, "Q", 10,
@@ -523,19 +526,7 @@ vips_foreign_save_jpeg_mime_init(VipsForeignSaveJpegMime *mime)
  * vips_jpegsave: (method)
  * @in: image to save
  * @filename: file to write to
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @Q: %gint, quality factor
- * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
- * * @interlace: %gboolean, write an interlaced (progressive) jpeg
- * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
- * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
- * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
- * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
- * * @quant_table: %gint, quantization table index
- * * @restart_interval: %gint, restart interval in mcu
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Write a VIPS image to a file as JPEG.
  *
@@ -568,17 +559,17 @@ vips_foreign_save_jpeg_mime_init(VipsForeignSaveJpegMime *mime)
  * If @quant_table is set and the version of libjpeg supports it
  * (e.g. mozjpeg >= 3.0) it selects the quantization table to use:
  *
- * * 0 — Tables from JPEG Annex K (vips and libjpeg default)
- * * 1 — Flat table
- * * 2 — Table tuned for MSSIM on Kodak image set
- * * 3 — Table from ImageMagick by N. Robidoux (current mozjpeg default)
- * * 4 — Table tuned for PSNR-HVS-M on Kodak image set
- * * 5 — Table from Relevance of Human Vision to JPEG-DCT Compression (1992)
- * * 6 — Table from DCTune Perceptual Optimization of Compressed Dental
+ * - 0 — Tables from JPEG Annex K (vips and libjpeg default)
+ * - 1 — Flat table
+ * - 2 — Table tuned for MSSIM on Kodak image set
+ * - 3 — Table from ImageMagick by N. Robidoux (current mozjpeg default)
+ * - 4 — Table tuned for PSNR-HVS-M on Kodak image set
+ * - 5 — Table from Relevance of Human Vision to JPEG-DCT Compression (1992)
+ * - 6 — Table from DCTune Perceptual Optimization of Compressed Dental
  *   X-Rays (1997)
- * * 7 — Table from A Visual Detection Model for DCT Coefficient
+ * - 7 — Table from A Visual Detection Model for DCT Coefficient
  *   Quantization (1993)
- * * 8 — Table from An Improved Detection Model for DCT Coefficient
+ * - 8 — Table from An Improved Detection Model for DCT Coefficient
  *   Quantization (1993)
  *
  * Quantization table 0 is the default in vips and libjpeg(-turbo), but it
@@ -604,15 +595,27 @@ vips_foreign_save_jpeg_mime_init(VipsForeignSaveJpegMime *mime)
  * The image is automatically converted to RGB, Monochrome or CMYK before
  * saving.
  *
- * EXIF data is constructed from #VIPS_META_EXIF_NAME, then
+ * EXIF data is constructed from [const@META_EXIF_NAME], then
  * modified with any other related tags on the image before being written to
- * the file. #VIPS_META_RESOLUTION_UNIT is used to set the EXIF resolution
- * unit. #VIPS_META_ORIENTATION is used to set the EXIF orientation tag.
+ * the file. [const@META_RESOLUTION_UNIT] is used to set the EXIF resolution
+ * unit. [const@META_ORIENTATION] is used to set the EXIF orientation tag.
  *
- * IPTC as #VIPS_META_IPTC_NAME and XMP as #VIPS_META_XMP_NAME
+ * IPTC as [const@META_IPTC_NAME] and XMP as [const@META_XMP_NAME]
  * are coded and attached.
  *
- * See also: vips_jpegsave_buffer(), vips_image_write_to_file().
+ * ::: tip "Optional arguments"
+ *     * @Q: `gint`, quality factor
+ *     * @optimize_coding: `gboolean`, compute optimal Huffman coding tables
+ *     * @interlace: `gboolean`, write an interlaced (progressive) jpeg
+ *     * @subsample_mode: [enum@ForeignSubsample], chroma subsampling mode
+ *     * @trellis_quant: `gboolean`, apply trellis quantisation to each 8x8 block
+ *     * @overshoot_deringing: `gboolean`, overshoot samples with extreme values
+ *     * @optimize_scans: `gboolean`, split DCT coefficients into separate scans
+ *     * @quant_table: `gint`, quantization table index
+ *     * @restart_interval: `gint`, restart interval in mcu
+ *
+ * ::: seealso
+ *     [method@Image.jpegsave_buffer], [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -633,23 +636,23 @@ vips_jpegsave(VipsImage *in, const char *filename, ...)
  * vips_jpegsave_target: (method)
  * @in: image to save
  * @target: save image to this target
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
+ * As [method@Image.jpegsave], but save to a target.
  *
- * * @Q: %gint, quality factor
- * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
- * * @interlace: %gboolean, write an interlaced (progressive) jpeg
- * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
- * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
- * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
- * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
- * * @quant_table: %gint, quantization table index
- * * @restart_interval: %gint, restart interval in mcu
+ * ::: tip "Optional arguments"
+ *     * @Q: `gint`, quality factor
+ *     * @optimize_coding: `gboolean`, compute optimal Huffman coding tables
+ *     * @interlace: `gboolean`, write an interlaced (progressive) jpeg
+ *     * @subsample_mode: [enum@ForeignSubsample], chroma subsampling mode
+ *     * @trellis_quant: `gboolean`, apply trellis quantisation to each 8x8 block
+ *     * @overshoot_deringing: `gboolean`, overshoot samples with extreme values
+ *     * @optimize_scans: `gboolean`, split DCT coefficients into separate scans
+ *     * @quant_table: `gint`, quantization table index
+ *     * @restart_interval: `gint`, restart interval in mcu
  *
- * As vips_jpegsave(), but save to a target.
- *
- * See also: vips_jpegsave(), vips_image_write_to_target().
+ * ::: seealso
+ *     [method@Image.jpegsave], [method@Image.write_to_target].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -671,27 +674,27 @@ vips_jpegsave_target(VipsImage *in, VipsTarget *target, ...)
  * @in: image to save
  * @buf: (array length=len) (element-type guint8): return output buffer here
  * @len: (type gsize): return output length here
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
- *
- * * @Q: %gint, quality factor
- * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
- * * @interlace: %gboolean, write an interlaced (progressive) jpeg
- * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
- * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
- * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
- * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
- * * @quant_table: %gint, quantization table index
- * * @restart_interval: %gint, restart interval in mcu
- *
- * As vips_jpegsave(), but save to a memory buffer.
+ * As [method@Image.jpegsave], but save to a memory buffer.
  *
  * The address of the buffer is returned in @obuf, the length of the buffer in
- * @olen. You are responsible for freeing the buffer with g_free() when you
- * are done with it.
+ * @olen. You are responsible for freeing the buffer with [func@GLib.free]
+ * when you are done with it.
  *
- * See also: vips_jpegsave(), vips_image_write_to_file().
+ * ::: tip "Optional arguments"
+ *     * @Q: `gint`, quality factor
+ *     * @optimize_coding: `gboolean`, compute optimal Huffman coding tables
+ *     * @interlace: `gboolean`, write an interlaced (progressive) jpeg
+ *     * @subsample_mode: [enum@ForeignSubsample], chroma subsampling mode
+ *     * @trellis_quant: `gboolean`, apply trellis quantisation to each 8x8 block
+ *     * @overshoot_deringing: `gboolean`, overshoot samples with extreme values
+ *     * @optimize_scans: `gboolean`, split DCT coefficients into separate scans
+ *     * @quant_table: `gint`, quantization table index
+ *     * @restart_interval: `gint`, restart interval in mcu
+ *
+ * ::: seealso
+ *     [method@Image.jpegsave], [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -726,23 +729,23 @@ vips_jpegsave_buffer(VipsImage *in, void **buf, size_t *len, ...)
 /**
  * vips_jpegsave_mime: (method)
  * @in: image to save
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
+ * As [method@Image.jpegsave], but save as a mime jpeg on stdout.
  *
- * * @Q: %gint, quality factor
- * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
- * * @interlace: %gboolean, write an interlaced (progressive) jpeg
- * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
- * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
- * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
- * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
- * * @quant_table: %gint, quantization table index
- * * @restart_interval: %gint, restart interval in mcu
+ * ::: tip "Optional arguments"
+ *     * @Q: `gint`, quality factor
+ *     * @optimize_coding: `gboolean`, compute optimal Huffman coding tables
+ *     * @interlace: `gboolean`, write an interlaced (progressive) jpeg
+ *     * @subsample_mode: [enum@ForeignSubsample], chroma subsampling mode
+ *     * @trellis_quant: `gboolean`, apply trellis quantisation to each 8x8 block
+ *     * @overshoot_deringing: `gboolean`, overshoot samples with extreme values
+ *     * @optimize_scans: `gboolean`, split DCT coefficients into separate scans
+ *     * @quant_table: `gint`, quantization table index
+ *     * @restart_interval: `gint`, restart interval in mcu
  *
- * As vips_jpegsave(), but save as a mime jpeg on stdout.
- *
- * See also: vips_jpegsave(), vips_image_write_to_file().
+ * ::: seealso
+ *     [method@Image.jpegsave], [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
