@@ -196,21 +196,17 @@ vips_foreign_load_jxl_build(VipsObject *object)
 	printf("vips_foreign_load_jxl_build:\n");
 #endif /*DEBUG*/
 
-	jxl->runner = JxlThreadParallelRunnerCreate(NULL,
-		vips_concurrency_get());
+	jxl->runner = JxlThreadParallelRunnerCreate(NULL, vips_concurrency_get());
 	jxl->decoder = JxlDecoderCreate(NULL);
 
 	if (JxlDecoderSetParallelRunner(jxl->decoder,
 			JxlThreadParallelRunner, jxl->runner)) {
-		vips_foreign_load_jxl_error(jxl,
-			"JxlDecoderSetParallelRunner");
+		vips_foreign_load_jxl_error(jxl, "JxlDecoderSetParallelRunner");
 		return -1;
 	}
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_jxl_parent_class)->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_jxl_parent_class)
+		->build(object);
 }
 
 static gboolean
@@ -439,6 +435,56 @@ vips_foreign_load_jxl_print_format(JxlPixelFormat *format)
 	printf("    endianness = %d\n", format->endianness);
 	printf("    align = %zd\n", format->align);
 }
+
+static const char *
+vips_foreign_load_jxl_blend_mode(JxlBlendMode blendmode)
+{
+	switch (blendmode) {
+	case JXL_BLEND_REPLACE:
+		return "JXL_BLEND_REPLACE";
+
+	case JXL_BLEND_ADD:
+		return "JXL_BLEND_ADD";
+
+	case JXL_BLEND_BLEND:
+		return "JXL_BLEND_BLEND";
+
+	case JXL_BLEND_MULADD:
+		return "JXL_BLEND_MULADD";
+
+	case JXL_BLEND_MUL:
+		return "JXL_BLEND_MUL";
+
+	default:
+		return "<unknown JxlBlendMode";
+	}
+}
+
+static void
+vips_foreign_load_jxl_print_frame_header(JxlFrameHeader *h)
+{
+	printf("JxlFrameHeader:\n");
+	printf("    duration = %u\n", h->duration);
+	printf("    timecode = %u\n", h->timecode);
+	printf("    name_length = %u\n", h->name_length);
+	printf("    is_last = %s\n", h->is_last ? "TRUE" : "FALSE");
+	printf("    layer_info.have_crop = %s\n",
+		h->layer_info.have_crop ? "TRUE" : "FALSE");
+	printf("    layer_info.crop_x0 = %d\n", h->layer_info.crop_x0);
+	printf("    layer_info.crop_y0 = %d\n", h->layer_info.crop_y0);
+	printf("    layer_info.xsize = %u\n", h->layer_info.xsize);
+	printf("    layer_info.ysize = %u\n", h->layer_info.ysize);
+	printf("    layer_info.blend_info.blendmode = %s\n",
+		vips_foreign_load_jxl_blend_mode(h->layer_info.blend_info.blendmode));
+	printf("    layer_info.blend_info.source = %u\n",
+		h->layer_info.blend_info.source);
+	printf("    layer_info.blend_info.alpha = %u\n",
+		h->layer_info.blend_info.alpha);
+	printf("    layer_info.blend_info.clamp = %s\n",
+		h->layer_info.blend_info.clamp ? "TRUE" : "FALSE");
+	printf("    layer_info.save_as_reference = %u\n",
+		h->layer_info.save_as_reference);
+}
 #endif /*DEBUG*/
 
 static JxlDecoderStatus
@@ -455,9 +501,9 @@ vips_foreign_load_jxl_process(VipsForeignLoadJxl *jxl)
 		size_t bytes_remaining;
 		int bytes_read;
 
-#ifdef DEBUG
+#ifdef DEBUG_VERBOSE
 		printf("vips_foreign_load_jxl_process: reading ...\n");
-#endif /*DEBUG*/
+#endif /*DEBUG_VERBOSE*/
 
 		bytes_remaining = JxlDecoderReleaseInput(jxl->decoder);
 		bytes_read = vips_foreign_load_jxl_fill_input(jxl, bytes_remaining);
@@ -498,6 +544,7 @@ vips_foreign_load_jxl_read_frame(VipsForeignLoadJxl *jxl, VipsImage *frame,
 #ifdef DEBUG_VERBOSE
 		printf("vips_foreign_load_jxl_read_frame: skipping %d frames\n", skip);
 #endif /*DEBUG_VERBOSE*/
+
 		JxlDecoderSkipFrames(jxl->decoder, skip);
 		jxl->frame_no += skip;
 	}
@@ -585,14 +632,12 @@ vips_foreign_load_jxl_generate(VipsRegion *out_region,
 static int
 vips_foreign_load_jxl_fix_exif(VipsForeignLoadJxl *jxl)
 {
-	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS(jxl);
-
 	if (!jxl->exif_data ||
 		vips_isprefix("Exif", (char *) jxl->exif_data))
 		return 0;
 
 	if (jxl->exif_size < 4) {
-		g_warning("%s: invalid data in EXIF box", class->nickname);
+		g_warning("invalid data in EXIF box");
 		return -1;
 	}
 
@@ -600,7 +645,7 @@ vips_foreign_load_jxl_fix_exif(VipsForeignLoadJxl *jxl)
 	 */
 	size_t offset = GUINT32_FROM_BE(*((guint32 *) jxl->exif_data));
 	if (offset > jxl->exif_size - 4) {
-		g_warning("%s: invalid data in EXIF box", class->nickname);
+		g_warning("invalid data in EXIF box");
 		return -1;
 	}
 
@@ -694,7 +739,7 @@ vips_foreign_load_jxl_set_header(VipsForeignLoadJxl *jxl, VipsImage *out)
 
 	if (jxl->frame_count > 1) {
 		if (jxl->n == -1)
-			jxl->n = jxl->frame_count - jxl->page;
+			jxl->n = jxl->frame_count - jxl->page; // FIXME: Invalidates operation cache
 
 		if (jxl->page < 0 ||
 			jxl->n <= 0 ||
@@ -715,14 +760,14 @@ vips_foreign_load_jxl_set_header(VipsForeignLoadJxl *jxl, VipsImage *out)
 
 			/* gif uses centiseconds for delays
 			 */
-			vips_image_set_int(out, "gif-delay", VIPS_RINT(delay[0] / 10.0));
+			vips_image_set_int(out, "gif-delay", rint(delay[0] / 10.0));
 
 			vips_image_set_int(out, "loop", jxl->info.animation.num_loops);
 		}
 	}
 	else {
-		jxl->n = 1;
-		jxl->page = 0;
+		jxl->n = 1; // FIXME: Invalidates operation cache
+		jxl->page = 0; // FIXME: Invalidates operation cache
 	}
 
 	/* Init jxl->frame only when we need to decode multiple frames.
@@ -822,8 +867,7 @@ vips_foreign_load_jxl_header(VipsForeignLoad *load)
 	do {
 		switch ((status = vips_foreign_load_jxl_process(jxl))) {
 		case JXL_DEC_ERROR:
-			vips_foreign_load_jxl_error(jxl,
-				"JxlDecoderProcessInput");
+			vips_foreign_load_jxl_error(jxl, "JxlDecoderProcessInput");
 			return -1;
 
 		case JXL_DEC_BOX:
@@ -907,8 +951,7 @@ vips_foreign_load_jxl_header(VipsForeignLoad *load)
 					&jxl->format,
 #endif
 					JXL_COLOR_PROFILE_TARGET_DATA, &jxl->icc_size)) {
-				vips_foreign_load_jxl_error(jxl,
-					"JxlDecoderGetICCProfileSize");
+				vips_foreign_load_jxl_error(jxl, "JxlDecoderGetICCProfileSize");
 				return -1;
 			}
 
@@ -937,12 +980,16 @@ vips_foreign_load_jxl_header(VipsForeignLoad *load)
 				return -1;
 			}
 
+#ifdef DEBUG
+			vips_foreign_load_jxl_print_frame_header(&h);
+#endif /*DEBUG*/
+
 			if (jxl->info.have_animation) {
 				// tick duration in seconds
 				double tick = (double) jxl->info.animation.tps_denominator /
 					jxl->info.animation.tps_numerator;
 				// this duration in ms
-				int ms = VIPS_RINT(1000.0 * h.duration * tick);
+				int ms = rint(1000.0 * h.duration * tick);
 				// h.duration of 0xffffffff is used for multipage JXL ... map
 				// this to -1 in delay
 				int duration = h.duration == 0xffffffff ? -1 : ms;
@@ -1010,8 +1057,7 @@ vips_foreign_load_jxl_load(VipsForeignLoad *load)
 
 	JxlDecoderRewind(jxl->decoder);
 	if (JxlDecoderSubscribeEvents(jxl->decoder,
-			JXL_DEC_FRAME |
-				JXL_DEC_FULL_IMAGE)) {
+				JXL_DEC_FRAME | JXL_DEC_FULL_IMAGE)) {
 		vips_foreign_load_jxl_error(jxl,
 			"JxlDecoderSubscribeEvents");
 		return -1;
@@ -1122,10 +1168,8 @@ vips_foreign_load_jxl_file_build(VipsObject *object)
 		!(jxl->source = vips_source_new_from_file(file->filename)))
 		return -1;
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_jxl_file_parent_class)->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_jxl_file_parent_class)
+		->build(object);
 }
 
 const char *vips__jxl_suffs[] = { ".jxl", NULL };
@@ -1202,10 +1246,8 @@ vips_foreign_load_jxl_buffer_build(VipsObject *object)
 				  VIPS_AREA(buffer->buf)->length)))
 			return -1;
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_jxl_file_parent_class)->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_jxl_buffer_parent_class)
+		->build(object);
 }
 
 static gboolean
@@ -1276,11 +1318,8 @@ vips_foreign_load_jxl_source_build(VipsObject *object)
 		g_object_ref(jxl->source);
 	}
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_jxl_source_parent_class)
-			->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_jxl_source_parent_class)
+		->build(object);
 }
 
 static void
