@@ -138,7 +138,7 @@ vips_foreign_save_raw_class_init(VipsForeignSaveRawClass *class)
 	object_class->description = _("save image to raw");
 	object_class->build = vips_foreign_save_raw_build;
 
-	save_class->saveable = VIPS_SAVEABLE_ANY;
+	save_class->saveable = VIPS_FOREIGN_SAVEABLE_ANY;
 }
 
 static void
@@ -327,12 +327,13 @@ vips_foreign_save_raw_buffer_init(VipsForeignSaveRawBuffer *buffer)
  * vips_rawsave: (method)
  * @in: image to save
  * @filename: file to write to
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Writes the pixels in @in to the file @filename with no header or other
  * metadata.
  *
- * See also: vips_image_write_to_file().
+ * ::: seealso
+ *     [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -354,15 +355,16 @@ vips_rawsave(VipsImage *in, const char *filename, ...)
  * @in: image to save
  * @buf: (array length=len) (element-type guint8): return output buffer here
  * @len: (type gsize): return output length here
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * As vips_rawsave(), but save to a memory buffer.
+ * As [method@Image.rawsave], but save to a memory buffer.
  *
  * The address of the buffer is returned in @buf, the length of the buffer in
- * @len. You are responsible for freeing the buffer with g_free() when you
+ * @len. You are responsible for freeing the buffer with [func@GLib.free] when you
  * are done with it.
  *
- * See also: vips_rawsave(), vips_image_write_to_memory(), vips_image_write_to_file().
+ * ::: seealso
+ *     [method@Image.rawsave], [method@Image.write_to_memory], [method@Image.write_to_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -398,11 +400,12 @@ vips_rawsave_buffer(VipsImage *in, void **buf, size_t *len, ...)
  * vips_rawsave_target: (method)
  * @in: image to save
  * @target: save image to this target
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * As vips_rawsave(), but save to a target.
+ * As [method@Image.rawsave], but save to a target.
  *
- * See also: vips_rawsave().
+ * ::: seealso
+ *     [method@Image.rawsave].
  *
  * Returns: 0 on success, -1 on error.
  */
