@@ -28,6 +28,8 @@
  * 	- support rsvg_handle_get_intrinsic_size_in_pixels()
  * 5/6/22
  * 	- allow random access
+ * 20/5/25
+ * 	- support high bitdepth rendering (128-bit)
  */
 
 /*
@@ -107,18 +109,26 @@ typedef struct _VipsForeignLoadSvg {
 	 */
 	double dpi;
 
-	/* Calculate this from DPI. At 72 DPI, we render 1:1 with cairo.
+	/* Scale by this factor.
 	 */
 	double scale;
 
-	/* Scale using cairo when SVG has no width and height attributes.
+	/* The total scale factor we render with.
 	 */
-	double cairo_scale;
+	double total_scale;
 
 	/* Allow SVGs of any size.
 	 */
 	gboolean unlimited;
 
+	/* Enables scRGB 128-bit output (32-bit per channel).
+	 */
+	gboolean high_bitdepth;
+
+	/* Custom CSS.
+	 */
+	const char *stylesheet;
+
 	RsvgHandle *page;
 
 } VipsForeignLoadSvg;
@@ -262,6 +272,7 @@ vips_foreign_load_svg_is_a(const void *buf, size_t len)
 		str[1] == '\213') {
 		z_stream zs;
 		size_t opos;
+		int err;
 
 		zs.zalloc = (alloc_func) vips_foreign_load_svg_zalloc;
 		zs.zfree = (free_func) vips_foreign_load_svg_zfree;
@@ -278,11 +289,18 @@ vips_foreign_load_svg_is_a(const void *buf, size_t len)
 		do {
 			zs.avail_out = sizeof(obuf) - opos;
 			zs.next_out = (unsigned char *) obuf + opos;
-			if (inflate(&zs, Z_NO_FLUSH) < Z_OK) {
+			err = inflate(&zs, Z_NO_FLUSH);
+
+			/* Always update opos after inflate(), even if stream ended.
+			 */
+			opos = sizeof(obuf) - zs.avail_out;
+
+			if (err == Z_STREAM_END)
+				break;
+			if (err != Z_OK) {
 				inflateEnd(&zs);
 				return FALSE;
 			}
-			opos = sizeof(obuf) - zs.avail_out;
 		} while (opos < sizeof(obuf) &&
 			zs.avail_in > 0);
 
@@ -325,6 +343,28 @@ vips_foreign_load_svg_dispose(GObject *gobject)
 	G_OBJECT_CLASS(vips_foreign_load_svg_parent_class)->dispose(gobject);
 }
 
+static int
+vips_foreign_load_svg_build(VipsObject *object)
+{
+	VipsForeignLoadSvg *svg G_GNUC_UNUSED = (VipsForeignLoadSvg *) object;
+
+#ifdef DEBUG
+	printf("vips_foreign_load_svg_build:\n");
+#endif /*DEBUG*/
+
+#ifndef HAVE_CAIRO_FORMAT_RGBA128F
+	if (svg->high_bitdepth) {
+		g_warning("ignoring high_bitdepth");
+		svg->high_bitdepth = FALSE;
+	}
+#endif /*HAVE_CAIRO_FORMAT_RGBA128F*/
+
+	svg->total_scale = svg->scale * svg->dpi / 72.0;
+
+	return VIPS_OBJECT_CLASS(vips_foreign_load_svg_parent_class)
+		->build(object);
+}
+
 static VipsForeignFlags
 vips_foreign_load_svg_get_flags_filename(const char *filename)
 {
@@ -341,7 +381,7 @@ vips_foreign_load_svg_get_flags(VipsForeignLoad *load)
 
 #if LIBRSVG_CHECK_VERSION(2, 52, 0)
 /* Derived from `CssLength::to_user` in librsvg.
- * https://gitlab.gnome.org/GNOME/librsvg/-/blob/e6607c9ae8d8409d4efff6b12993717400b3356e/src/length.rs#L368
+ * https://gitlab.gnome.org/GNOME/librsvg/-/blob/2.60.0/rsvg/src/length.rs#L403
  */
 static double
 svg_css_length_to_pixels(RsvgLength length, double dpi)
@@ -516,18 +556,14 @@ vips_foreign_load_svg_get_scaled_size(VipsForeignLoadSvg *svg,
 	double width;
 	double height;
 
-	/* Get dimensions with the default dpi.
+	/* Set target DPI to scale non-pixel units correctly.
 	 */
-	rsvg_handle_set_dpi(svg->page, 72.0);
+	rsvg_handle_set_dpi(svg->page, svg->dpi);
 	if (vips_foreign_load_svg_get_natural_size(svg, &width, &height))
 		return -1;
 
-	/* We scale up with cairo --- scaling with rsvg_handle_set_dpi() will
-	 * fail for SVGs with absolute sizes.
-	 */
-	svg->cairo_scale = svg->scale * svg->dpi / 72.0;
-	width *= svg->cairo_scale;
-	height *= svg->cairo_scale;
+	width *= svg->total_scale;
+	height *= svg->total_scale;
 
 	*out_width = VIPS_ROUND_UINT(width);
 	*out_height = VIPS_ROUND_UINT(height);
@@ -538,21 +574,31 @@ vips_foreign_load_svg_get_scaled_size(VipsForeignLoadSvg *svg,
 static int
 vips_foreign_load_svg_parse(VipsForeignLoadSvg *svg, VipsImage *out)
 {
+	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS(svg);
+
 	int width;
 	int height;
 	double res;
 
 	if (vips_foreign_load_svg_get_scaled_size(svg, &width, &height))
 		return -1;
+	if (width <= 0 ||
+		height <= 0) {
+		vips_error(class->nickname, "%s", _("zero-sized image"));
+		return -1;
+	}
 
 	/* We need pixels/mm for vips.
 	 */
 	res = svg->dpi / 25.4;
 
 	vips_image_init_fields(out,
-		width, height,
-		4, VIPS_FORMAT_UCHAR,
-		VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, res, res);
+		width, height, 4,
+		svg->high_bitdepth ? VIPS_FORMAT_FLOAT : VIPS_FORMAT_UCHAR,
+		VIPS_CODING_NONE,
+		svg->high_bitdepth ?
+			VIPS_INTERPRETATION_scRGB : VIPS_INTERPRETATION_sRGB,
+		res, res);
 
 	/* We use a tilecache, so it's smalltile.
 	 */
@@ -593,14 +639,37 @@ vips_foreign_load_svg_generate(VipsRegion *out_region,
 	 */
 	vips_region_black(out_region);
 
+#ifdef HAVE_CAIRO_FORMAT_RGBA128F
+	cairo_format_t format =
+		svg->high_bitdepth ? CAIRO_FORMAT_RGBA128F : CAIRO_FORMAT_ARGB32;
+#else
+	cairo_format_t format = CAIRO_FORMAT_ARGB32;
+#endif /*HAVE_CAIRO_FORMAT_RGBA128F*/
+
 	surface = cairo_image_surface_create_for_data(
 		VIPS_REGION_ADDR(out_region, r->left, r->top),
-		CAIRO_FORMAT_ARGB32,
+		format,
 		r->width, r->height,
 		VIPS_REGION_LSKIP(out_region));
 	cr = cairo_create(surface);
 	cairo_surface_destroy(surface);
 
+#if LIBRSVG_CHECK_VERSION(2, 48, 0)
+	if (svg->stylesheet &&
+		g_utf8_validate(svg->stylesheet, -1, NULL)) {
+		GError *error = NULL;
+		if (!rsvg_handle_set_stylesheet(svg->page,
+				(const guint8 *) svg->stylesheet,
+				g_utf8_strlen(svg->stylesheet, -1), &error)) {
+			cairo_destroy(cr);
+			vips_operation_invalidate(VIPS_OPERATION(svg));
+			vips_error(class->nickname, "Invalid custom CSS");
+			vips_g_error(&error);
+			return -1;
+		}
+	}
+#endif
+
 	/* rsvg is single-threaded, but we don't need to lock since we're
 	 * running inside a non-threaded tilecache.
 	 */
@@ -610,15 +679,16 @@ vips_foreign_load_svg_generate(VipsRegion *out_region,
 		RsvgRectangle viewport;
 		GError *error = NULL;
 
-		/* No need to scale -- we always set the viewport to the
-		 * whole image, and set the region to draw on the surface.
-		 */
-		cairo_translate(cr, -r->left, -r->top);
 		viewport.x = 0;
 		viewport.y = 0;
 		viewport.width = out_region->im->Xsize;
 		viewport.height = out_region->im->Ysize;
 
+		/* No need to scale -- we always set the viewport to the
+		 * whole image, and set the region to draw on the surface.
+		 */
+		cairo_translate(cr, -r->left, -r->top);
+
 		if (!rsvg_handle_render_document(svg->page, cr, &viewport, &error)) {
 			cairo_destroy(cr);
 			vips_operation_invalidate(VIPS_OPERATION(svg));
@@ -633,9 +703,9 @@ vips_foreign_load_svg_generate(VipsRegion *out_region,
 
 #else /*!LIBRSVG_CHECK_VERSION(2, 46, 0)*/
 
-	cairo_scale(cr, svg->cairo_scale, svg->cairo_scale);
-	cairo_translate(cr, -r->left / svg->cairo_scale,
-		-r->top / svg->cairo_scale);
+	cairo_scale(cr, svg->total_scale, svg->total_scale);
+	cairo_translate(cr, -r->left / svg->total_scale,
+		-r->top / svg->total_scale);
 
 	if (!rsvg_handle_render_cairo(svg->page, cr)) {
 		cairo_destroy(cr);
@@ -648,13 +718,23 @@ vips_foreign_load_svg_generate(VipsRegion *out_region,
 	cairo_destroy(cr);
 
 #endif /*LIBRSVG_CHECK_VERSION(2, 46, 0)*/
-
-	/* Cairo makes pre-multipled BRGA -- we must byteswap and unpremultiply.
-	 */
-	for (y = 0; y < r->height; y++)
-		vips__premultiplied_bgra2rgba(
-			(guint32 *) VIPS_REGION_ADDR(out_region, r->left, r->top + y),
-			r->width);
+	if (svg->high_bitdepth) {
+		/* Assuming the surface is RGBA128F and the data is premultiplied.
+		   Loop through each row and unpremultiply the float data.
+		*/
+		for (y = 0; y < r->height; y++)
+			vips__premultiplied_rgb1282scrgba(
+				(float *) VIPS_REGION_ADDR(out_region, r->left, r->top + y),
+				r->width);
+	}
+	else {
+		/* Cairo makes pre-multipled BRGA -- we must byteswap and unpremultiply.
+		 */
+		for (y = 0; y < r->height; y++)
+			vips__premultiplied_bgra2rgba(
+				(guint32 *) VIPS_REGION_ADDR(out_region, r->left, r->top + y),
+				r->width);
+	}
 
 	return 0;
 }
@@ -698,6 +778,7 @@ vips_foreign_load_svg_class_init(VipsForeignLoadSvgClass *class)
 
 	object_class->nickname = "svgload_base";
 	object_class->description = _("load SVG with rsvg");
+	object_class->build = vips_foreign_load_svg_build;
 
 	/* librsvg has not been fuzzed, so should not be used with
 	 * untrusted input unless you are very careful.
@@ -735,6 +816,20 @@ vips_foreign_load_svg_class_init(VipsForeignLoadSvgClass *class)
 		G_STRUCT_OFFSET(VipsForeignLoadSvg, unlimited),
 		FALSE);
 #endif
+
+	VIPS_ARG_STRING(class, "stylesheet", 24,
+		_("Stylesheet"),
+		_("Custom CSS"),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET(VipsForeignLoadSvg, stylesheet),
+		NULL);
+
+	VIPS_ARG_BOOL(class, "high_bitdepth", 25,
+		_("High bitdepth"),
+		_("Enable scRGB 128-bit output (32-bit per channel)"),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET(VipsForeignLoadSvg, high_bitdepth),
+		FALSE);
 }
 
 static void
@@ -742,7 +837,6 @@ vips_foreign_load_svg_init(VipsForeignLoadSvg *svg)
 {
 	svg->dpi = 72.0;
 	svg->scale = 1.0;
-	svg->cairo_scale = 1.0;
 }
 
 typedef struct _VipsForeignLoadSvgSource {
@@ -1012,19 +1106,14 @@ vips_foreign_load_svg_buffer_init(VipsForeignLoadSvgBuffer *buffer)
  * vips_svgload:
  * @filename: file to load
  * @out: (out): output image
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
+ * Render a SVG file into a VIPS image.
  *
- * * @dpi: %gdouble, render at this DPI
- * * @scale: %gdouble, scale render by this factor
- * * @unlimited: %gboolean, allow SVGs of any size
+ * Rendering uses the librsvg library and should be fast.
  *
- * Render a SVG file into a VIPS image.  Rendering uses the librsvg library
- * and should be fast.
- *
- * Use @dpi to set the rendering resolution. The default is 72. You can also
- * scale the rendering by @scale.
+ * Use @dpi to set the rendering resolution. The default is 72. Additionally,
+ * you can scale by setting @scale. If you set both, they combine.
  *
  * This function only reads the image header and does not render any pixel
  * data. Rendering occurs when pixels are accessed.
@@ -1032,7 +1121,22 @@ vips_foreign_load_svg_buffer_init(VipsForeignLoadSvgBuffer *buffer)
  * SVGs larger than 10MB are normally blocked for security. Set @unlimited to
  * allow SVGs of any size.
  *
- * See also: vips_image_new_from_file().
+ * A UTF-8 string containing custom CSS can be provided via @stylesheet.
+ * During the CSS cascade, the specified stylesheet will be applied with a
+ * User Origin. This feature requires librsvg 2.48.0 or later.
+ *
+ * If @high_bitdepth is set and the version of cairo supports it
+ * (e.g. cairo >= 1.17.2), enable 128-bit scRGB output (32-bit per channel).
+ *
+ * ::: tip "Optional arguments"
+ *     * @dpi: `gdouble`, render at this DPI
+ *     * @scale: `gdouble`, scale render by this factor
+ *     * @unlimited: `gboolean`, allow SVGs of any size
+ *     * @stylesheet: `gchararray`, custom CSS
+ *     * @high_bitdepth: `gboolean`, enable scRGB 128-bit output
+ *
+ * ::: seealso
+ *     [ctor@Image.new_from_file].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -1054,21 +1158,23 @@ vips_svgload(const char *filename, VipsImage **out, ...)
  * @buf: (array length=len) (element-type guint8): memory area to load
  * @len: (type gsize): size of memory area
  * @out: (out): image to write
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @dpi: %gdouble, render at this DPI
- * * @scale: %gdouble, scale render by this factor
- * * @unlimited: %gboolean, allow SVGs of any size
+ * @...: `NULL`-terminated list of optional named arguments
  *
  * Read a SVG-formatted memory block into a VIPS image. Exactly as
- * vips_svgload(), but read from a memory buffer.
+ * [ctor@Image.svgload], but read from a memory buffer.
  *
  * You must not free the buffer while @out is active. The
- * #VipsObject::postclose signal on @out is a good place to free.
+ * [signal@Object::postclose] signal on @out is a good place to free.
  *
- * See also: vips_svgload().
+ * ::: tip "Optional arguments"
+ *     * @dpi: `gdouble`, render at this DPI
+ *     * @scale: `gdouble`, scale render by this factor
+ *     * @unlimited: `gboolean`, allow SVGs of any size
+ *     * @stylesheet: `gchararray`, custom CSS
+ *     * @high_bitdepth: `gboolean`, enable scRGB 128-bit output
+ *
+ * ::: seealso
+ *     [ctor@Image.svgload].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -1096,18 +1202,20 @@ vips_svgload_buffer(void *buf, size_t len, VipsImage **out, ...)
  * vips_svgload_string:
  * @str: string to load
  * @out: (out): image to write
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Optional arguments:
+ * Exactly as [ctor@Image.svgload], but read from a string. This function
+ * takes a copy of the string.
  *
- * * @dpi: %gdouble, render at this DPI
- * * @scale: %gdouble, scale render by this factor
- * * @unlimited: %gboolean, allow SVGs of any size
+ * ::: tip "Optional arguments"
+ *     * @dpi: `gdouble`, render at this DPI
+ *     * @scale: `gdouble`, scale render by this factor
+ *     * @unlimited: `gboolean`, allow SVGs of any size
+ *     * @stylesheet: `gchararray`, custom CSS
+ *     * @high_bitdepth: `gboolean`, enable scRGB 128-bit output
  *
- * Exactly as vips_svgload(), but read from a string. This function takes a
- * copy of the string.
- *
- * See also: vips_svgload().
+ * ::: seealso
+ *     [ctor@Image.svgload].
  *
  * Returns: 0 on success, -1 on error.
  */
@@ -1135,11 +1243,12 @@ vips_svgload_string(const char *str, VipsImage **out, ...)
  * vips_svgload_source:
  * @source: source to load from
  * @out: (out): image to write
- * @...: %NULL-terminated list of optional named arguments
+ * @...: `NULL`-terminated list of optional named arguments
  *
- * Exactly as vips_svgload(), but read from a source.
+ * Exactly as [ctor@Image.svgload], but read from a source.
  *
- * See also: vips_svgload().
+ * ::: seealso
+ *     [ctor@Image.svgload].
  *
  * Returns: 0 on success, -1 on error.
  */
