@@ -27,6 +27,14 @@ class TestForeign:
         im = pyvips.Image.new_from_file(GIF_FILE)
         cls.onebit = im[1] > 128
 
+        all = [cls.mono, cls.colour, cls.cmyk]
+        # and alpha variants of all of them
+        alpha = [x.bandjoin(255) for x in all]
+        # and with a second alpha
+        alpha2 = [x.bandjoin(255) for x in alpha]
+
+        cls.all = all + alpha + alpha2
+
     @classmethod
     def teardown_class(cls):
         shutil.rmtree(cls.tempdir, ignore_errors=True)
@@ -35,6 +43,7 @@ class TestForeign:
         cls.mono = None
         cls.cmyk = None
         cls.onebit = None
+        cls.all = None
 
     # we have test files for formats which have a clear standard
     def file_loader(self, loader, test_file, validate):
@@ -148,6 +157,10 @@ class TestForeign:
         self.save_load_buffer("jpegsave_buffer", "jpegload_buffer",
                               self.colour, 80)
 
+        for image in self.all:
+            target = pyvips.Target.new_to_memory()
+            image.jpegsave_target(target)
+
         # see if we have exif parsing: our test image has this field
         x = pyvips.Image.new_from_file(JPEG_FILE)
         if x.get_typeof("exif-ifd0-Orientation") != 0:
@@ -422,6 +435,10 @@ class TestForeign:
         self.save_load_file(".png", "[interlace]", self.colour)
         self.save_load_file(".png", "[interlace]", self.mono)
 
+        for image in self.all:
+            target = pyvips.Target.new_to_memory()
+            image.pngsave_target(target)
+
         def png_indexed_valid(im):
             a = im(10, 10)
             assert_almost_equal_objects(a, [148.0, 131.0, 109.0])
@@ -432,7 +449,8 @@ class TestForeign:
             assert im.get("palette") == 1
 
         self.file_loader("pngload", PNG_INDEXED_FILE, png_indexed_valid)
-        self.buffer_loader("pngload_buffer", PNG_INDEXED_FILE, png_indexed_valid)
+        self.buffer_loader("pngload_buffer",
+            PNG_INDEXED_FILE, png_indexed_valid)
 
         # size of a regular mono PNG
         len_mono = len(self.mono.write_to_buffer(".png"))
@@ -482,7 +500,32 @@ class TestForeign:
             "exif-ifd0-ImageDescription", "test description")
         im2 = pyvips.Image.new_from_buffer(
             im1.write_to_buffer(".png"), "")
-        assert im2.get("exif-ifd0-ImageDescription").startswith("test description")
+        assert im2.get("exif-ifd0-ImageDescription") \
+            .startswith("test description")
+
+        # https://github.com/libvips/libvips/issues/4509
+        data = self.rgba.cast("double").write_to_buffer(".png")
+        after = pyvips.Image.new_from_buffer(data, "")
+        assert (self.rgba - after).abs().max() == 0
+
+        # we should be able to save an 8-bit image as a 16-bit PNG
+        data = self.colour.pngsave_buffer(bitdepth=16)
+        rgb16 = pyvips.Image.pngload_buffer(data)
+        assert rgb16.format == "ushort"
+
+        # we should be able to save a 16-bit image as an 8-bit PNG
+        data = rgb16.pngsave_buffer(bitdepth=8)
+        rgb = pyvips.Image.pngload_buffer(data)
+        assert rgb.format == "uchar"
+
+        # we should be able to save a 16-bit image as an 8-bit WebP
+        if have("webpsave"):
+            data = rgb16.webpsave_buffer(lossless=True)
+            rgb = pyvips.Image.webpload_buffer(data)
+            assert rgb.format == "uchar"
+            # ... and check if it was correctly shifted down
+            # https://github.com/libvips/libvips/issues/4568
+            assert (self.colour - rgb).abs().max() == 0
 
     @skip_if_no("tiffload")
     def test_tiff(self):
@@ -497,6 +540,10 @@ class TestForeign:
         self.file_loader("tiffload", TIF_FILE, tiff_valid)
         self.buffer_loader("tiffload_buffer", TIF_FILE, tiff_valid)
 
+        for image in self.all:
+            target = pyvips.Target.new_to_memory()
+            image.tiffsave_target(target)
+
         def tiff1_valid(im):
             a = im(127, 0)
             assert_almost_equal_objects(a, [0.0])
@@ -533,28 +580,6 @@ class TestForeign:
 
         self.file_loader("tiffload", TIF4_FILE, tiff4_valid)
 
-        def tiff_ojpeg_tile_valid(im):
-            a = im(10, 10)
-            assert_almost_equal_objects(a, [135.0, 156.0, 177.0, 255.0])
-            assert im.width == 234
-            assert im.height == 213
-            assert im.bands == 4
-            assert im.get("bits-per-sample") == 8
-
-        self.file_loader("tiffload", TIF_OJPEG_TILE_FILE, tiff_ojpeg_tile_valid)
-        self.buffer_loader("tiffload_buffer", TIF_OJPEG_TILE_FILE, tiff_ojpeg_tile_valid)
-
-        def tiff_ojpeg_strip_valid(im):
-            a = im(10, 10)
-            assert_almost_equal_objects(a, [228.0, 15.0, 9.0, 255.0])
-            assert im.width == 160
-            assert im.height == 160
-            assert im.bands == 4
-            assert im.get("bits-per-sample") == 8
-
-        self.file_loader("tiffload", TIF_OJPEG_STRIP_FILE, tiff_ojpeg_strip_valid)
-        self.buffer_loader("tiffload_buffer", TIF_OJPEG_STRIP_FILE, tiff_ojpeg_strip_valid)
-
         def tiff_subsampled_valid(im):
             a = im(10, 10)
             assert_almost_equal_objects(a, [6.0, 5.0, 21.0, 255.0])
@@ -662,6 +687,25 @@ class TestForeign:
         assert x1.xres == 100
         assert x1.yres == 200
 
+        filename = temp_filename(self.tempdir, '.tif')
+        x = pyvips.Image.new_from_file(TIF_FILE)
+        x = x.copy(xres=100, yres=200)
+        x.remove("resolution-unit")
+        x.write_to_file(filename)
+        x1 = pyvips.Image.new_from_file(filename)
+        assert x1.get("resolution-unit") == "in"
+        assert x1.xres == 100
+        assert x1.yres == 200
+
+        if sys.platform == "darwin":
+            with open(TIF2_FILE, 'rb') as f:
+                buf = bytearray(f.read())
+            buf = buf[:-4]
+            source = pyvips.Source.new_from_memory(buf)
+            im = pyvips.Image.tiffload_source(source, fail_on="warning")
+            with pytest.raises(Exception) as e_info:
+                im.avg() > 0
+
         # OME support in 8.5
         x = pyvips.Image.new_from_file(OME_FILE)
         assert x.width == 439
@@ -725,6 +769,31 @@ class TestForeign:
             z = y.hist_find(band=0)
             assert z(0, 0)[0] + z(255, 0)[0] == y.width * y.height
 
+    @skip_if_no("tiffload")
+    @pytest.mark.xfail(raises=AssertionError, reason="fails when libtiff was configured with --disable-old-jpeg")
+    def test_tiff_ojpeg(self):
+        def tiff_ojpeg_tile_valid(im):
+            a = im(10, 10)
+            assert_almost_equal_objects(a, [135.0, 156.0, 177.0, 255.0])
+            assert im.width == 234
+            assert im.height == 213
+            assert im.bands == 4
+            assert im.get("bits-per-sample") == 8
+
+        self.file_loader("tiffload", TIF_OJPEG_TILE_FILE, tiff_ojpeg_tile_valid)
+        self.buffer_loader("tiffload_buffer", TIF_OJPEG_TILE_FILE, tiff_ojpeg_tile_valid)
+
+        def tiff_ojpeg_strip_valid(im):
+            a = im(10, 10)
+            assert_almost_equal_objects(a, [228.0, 15.0, 9.0, 255.0])
+            assert im.width == 160
+            assert im.height == 160
+            assert im.bands == 4
+            assert im.get("bits-per-sample") == 8
+
+        self.file_loader("tiffload", TIF_OJPEG_STRIP_FILE, tiff_ojpeg_strip_valid)
+        self.buffer_loader("tiffload_buffer", TIF_OJPEG_STRIP_FILE, tiff_ojpeg_strip_valid)
+
     @skip_if_no("jp2kload")
     @skip_if_no("tiffload")
     def test_tiffjp2k(self):
@@ -771,16 +840,14 @@ class TestForeign:
         assert im.height == height * 5
 
         # page/n let you pick a range of pages
-        # 'n' param added in 8.5
-        if pyvips.at_least_libvips(8, 5):
-            im = pyvips.Image.magickload(GIF_ANIM_FILE)
-            width = im.width
-            height = im.height
-            im = pyvips.Image.magickload(GIF_ANIM_FILE, page=1, n=2)
-            assert im.width == width
-            assert im.height == height * 2
-            page_height = im.get("page-height")
-            assert page_height == height
+        im = pyvips.Image.magickload(GIF_ANIM_FILE)
+        width = im.width
+        height = im.height
+        im = pyvips.Image.magickload(GIF_ANIM_FILE, page=1, n=2)
+        assert im.width == width
+        assert im.height == height * 2
+        page_height = im.get("page-height")
+        assert page_height == height
 
         # should work for dicom
         im = pyvips.Image.magickload(DICOM_FILE)
@@ -1003,6 +1070,7 @@ class TestForeign:
 
     @skip_if_no("openslideload")
     def test_openslideload(self):
+
         def openslide_valid(im):
             a = im(10, 10)
             assert_almost_equal_objects(a, [244, 250, 243, 255])
@@ -1058,7 +1126,9 @@ class TestForeign:
         assert x2.get("bits-per-sample") == 4
         assert x2.get("palette") == 1
 
-        x2 = pyvips.Image.new_from_file(GIF_ANIM_FILE, n=-1)
+        x2 = pyvips.Image.new_from_file(GIF_ANIM_FILE,
+                                        n=-1,
+                                        access="sequential")
         # our test gif has delay 0 for the first frame set in error
         assert x2.get("delay") == [0, 50, 50, 50, 50]
         assert x2.get("loop") == 32761
@@ -1189,12 +1259,25 @@ class TestForeign:
         assert im.width == 10
         assert im.height == 10
 
+        # Custom CSS stylesheet
+        im = pyvips.Image.new_from_file(SVG_FILE)
+        assert im.avg() < 5
+        im = pyvips.Image.new_from_file(SVG_FILE, stylesheet=b'path{stroke:#f00;stroke-width:1em;}')
+        assert im.avg() > 5
+
+        with pytest.raises(pyvips.error.Error):
+            im = pyvips.Image.new_from_file(TRUNCATED_SVGZ_FILE)
+
     def test_csv(self):
         self.save_load("%s.csv", self.mono)
 
     def test_matrix(self):
         self.save_load("%s.mat", self.mono)
 
+        for image in self.all:
+            target = pyvips.Target.new_to_memory()
+            image.matrixsave_target(target)
+
     @skip_if_no("ppmload")
     def test_ppm(self):
         self.save_load("%s.ppm", self.colour)
@@ -1212,8 +1295,7 @@ class TestForeign:
         self.save_load_file("%s.ppm", "[ascii]", grey16, 0)
         self.save_load_file("%s.ppm", "[ascii]", rgb16, 0)
 
-        source = pyvips.Source.new_from_memory(b'P1\n#\n#\n1 1\n0\n')
-        im = pyvips.Image.ppmload_source(source)
+        im = pyvips.Image.new_from_buffer(b'P1\n#\n#\n1 1\n0\n', "")
         assert im.height == 1
         assert im.width == 1
 
@@ -1334,10 +1416,36 @@ class TestForeign:
 
         # 256x256 tiles, no overlap
         assert os.path.exists(filename + "/ImageProperties.xml")
+        with open(filename + "/ImageProperties.xml", 'rb') as f:
+            buf = f.read()
+        assert buf == (b'<IMAGE_PROPERTIES WIDTH="290" HEIGHT="442" '
+                       b'NUMTILES="5" NUMIMAGES="1" VERSION="1.8" '
+                       b'TILESIZE="256" />\n')
         x = pyvips.Image.new_from_file(filename + "/TileGroup0/1-0-0.jpg")
         assert x.width == 256
         assert x.height == 256
 
+        # IIIF v2
+        im = pyvips.Image.black(3509, 2506, bands=3)
+        filename = temp_filename(self.tempdir, '')
+        im.dzsave(filename, layout="iiif")
+        assert os.path.exists(filename + "/info.json")
+        assert os.path.exists(filename + "/0,0,512,512/512,/0/default.jpg")
+        assert os.path.exists(filename + "/2560,2048,512,458/512,/0/default.jpg")
+        x = pyvips.Image.new_from_file(filename + "/full/439,/0/default.jpg")
+        assert x.width == 439
+        assert x.height == 314
+
+        # IIIF v3
+        filename = temp_filename(self.tempdir, '')
+        im.dzsave(filename, layout="iiif3")
+        assert os.path.exists(filename + "/info.json")
+        assert os.path.exists(filename + "/0,0,512,512/512,512/0/default.jpg")
+        assert os.path.exists(filename + "/2560,2048,512,458/512,458/0/default.jpg")
+        x = pyvips.Image.new_from_file(filename + "/full/439,314/0/default.jpg")
+        assert x.width == 439
+        assert x.height == 314
+
         # test zip output
         filename = temp_filename(self.tempdir, '.zip')
         self.colour.dzsave(filename)
@@ -1403,7 +1511,7 @@ class TestForeign:
         # test keep=pyvips.ForeignKeep.ICC ... icc profiles should be
         # passed down
         filename = temp_filename(self.tempdir, '')
-        self.colour.dzsave(filename, keep=1 << 3) # pyvips.ForeignKeep.ICC
+        self.colour.dzsave(filename, keep=pyvips.ForeignKeep.ICC)
 
         y = pyvips.Image.new_from_file(filename + "_files/0/0_0.jpeg")
         assert y.get_typeof("icc-profile-data") != 0
@@ -1481,6 +1589,7 @@ class TestForeign:
             assert y.get("exif-ifd0-XPComment").startswith("banana")
 
     @skip_if_no("heifsave")
+    @pytest.mark.xfail(raises=pyvips.error.Error, reason="requires libheif built with patent-encumbered HEVC dependencies")
     def test_heicsave_16_to_12(self):
         rgb16 = self.colour.colourspace("rgb16")
         data = rgb16.heifsave_buffer(lossless=True)
@@ -1494,6 +1603,7 @@ class TestForeign:
         assert((im - rgb16).abs().max() < 4500)
 
     @skip_if_no("heifsave")
+    @pytest.mark.xfail(raises=pyvips.error.Error, reason="requires libheif built with patent-encumbered HEVC dependencies")
     def test_heicsave_16_to_8(self):
         rgb16 = self.colour.colourspace("rgb16")
         data = rgb16.heifsave_buffer(lossless=True, bitdepth=8)
@@ -1507,6 +1617,7 @@ class TestForeign:
         assert((im - rgb16 / 256).abs().max() < 80)
 
     @skip_if_no("heifsave")
+    @pytest.mark.xfail(raises=pyvips.error.Error, reason="requires libheif built with patent-encumbered HEVC dependencies")
     def test_heicsave_8_to_16(self):
         data = self.colour.heifsave_buffer(lossless=True, bitdepth=12)
         im = pyvips.Image.heifload_buffer(data)
@@ -1634,6 +1745,7 @@ class TestForeign:
         lossless = self.colour.jxlsave_buffer(lossless=True)
         assert len(lossy) < len(lossless) / 5
 
+    @skip_if_no("gifload")
     @skip_if_no("gifsave")
     def test_gifsave(self):
         # Animated GIF round trip
